Module: SuperSettings::SettingsHelper
- Defined in:
- app/helpers/super_settings/settings_helper.rb
Instance Method Summary collapse
-
#super_settings_application_header ⇒ String
Render the header for the web pages using values set in the configuration.
-
#super_settings_application_name ⇒ Object
Return the application name set by the configuration or a default value.
-
#super_settings_layout_style_tag ⇒ Object
Render the styles.css as an inline